@@ -1,5 +1,5 @@
 /* Find the length of STRING, but scan at most MAXLEN characters.
-   Copyright (C) 1996, 1997, 1998, 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998, 2000-2003 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    This program is free software; you can redistribute it and/or modify
@@ -12,37 +12,24 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #if HAVE_CONFIG_H
 # include <config.h>
 #endif
+#undef strnlen
 
-#if HAVE_STRING_H
-# if !STDC_HEADERS && HAVE_MEMORY_H
-#  include <memory.h>
-# endif
-/* Temporarily redefine strnlen so that an inconsistent prototype
-   (on at least AIX4.3.2.0 w/gcc-2.95.3) doesn't cause trouble.  */
-# define strnlen system_strnlen
-# include <string.h>
-# undef strnlen
-#else
-# include <strings.h>
-#endif
-
-#ifndef HAVE_DECL_MEMCHR
-"this configure-time declaration test was not run"
-#endif
-#if !HAVE_DECL_MEMCHR
-char *memchr ();
-#endif
+#include <string.h>
 
 #undef __strnlen
 #undef strnlen
 
+#ifndef _LIBC
+# define strnlen rpl_strnlen
+#endif
+
 #ifndef weak_alias
 # define __strnlen strnlen
 #endif
