@@ -1,5 +1,5 @@
 /* Subprocesses with pipes.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -15,12 +15,16 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to the Free
-   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 /* Written by Paul Eggert <eggert@twinsun.com>
    and Florian Krohm <florian@edamail.fishkill.ibm.com>.  */
 
+#if HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
 void init_subpipe (void);
 pid_t create_subpipe (char const * const *, int[2]);
 void reap_subpipe (pid_t, char const *);
