@@ -15,14 +15,14 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef LR0_H_
 # define LR0_H_
 
 # include "state.h"
 
-void generate_states PARAMS ((void));
+void generate_states (void);
 
 #endif /* !LR0_H_ */
