@@ -15,17 +15,18 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "system.h"
+
 #include "assoc.h"
 
 
 const char *
-assoc_to_string (assoc_t assoc)
+assoc_to_string (assoc a)
 {
-  switch (assoc)
+  switch (a)
     {
     case undef_assoc:
       return "undefined associativity";
