@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef ASSOC_H_
 # define ASSOC_H_
@@ -28,8 +28,8 @@ typedef enum
   right_assoc,
   left_assoc,
   non_assoc
-} assoc_t;
+} assoc;
 
-const char *assoc_to_string PARAMS ((assoc_t assoc));
+char const *assoc_to_string (assoc);
 
 #endif /* !ASSOC_H_ */
