@@ -1,5 +1,7 @@
 /* System-dependent definitions for Bison.
-   Copyright (C) 2000, 2001, 2002 Free Software Foundation, Inc.
+
+   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 Free Software
+   Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +15,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef BISON_SYSTEM_H
 #define BISON_SYSTEM_H
@@ -22,8 +24,11 @@
 # include <config.h>
 #endif
 
+#include <limits.h>
 #include <stddef.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 /* Verify a requirement at compile-time (unlike assert, which is runtime).  */
 #define verify(name, assertion) struct name {char name[(assertion) ? 1 : -1];}
@@ -32,90 +37,31 @@
 # include <sys/types.h>
 #endif
 
-#if HAVE_STDLIB_H
-# include <stdlib.h>
-#endif
-
-/* The following test is to work around the gross typo in
-   systems like Sony NEWS-OS Release 4.0C, whereby EXIT_FAILURE
-   is defined to 0, not 1.  */
-#if !EXIT_FAILURE
-# undef EXIT_FAILURE
-# define EXIT_FAILURE 1
-#endif
-
-#ifndef EXIT_SUCCESS
-# define EXIT_SUCCESS 0
-#endif
-
 #if HAVE_UNISTD_H
 # include <unistd.h>
 #endif
 
-#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
-# include <string.h>
-/* An ANSI string.h and pre-ANSI memory.h might conflict.  */
-# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#  include <memory.h>
-# endif /* not STDC_HEADERS and HAVE_MEMORY_H */
-#else /* not STDC_HEADERS and not HAVE_STRING_H */
-# include <strings.h>
-/* memory.h and strings.h conflict on some systems.  */
-#endif /* not STDC_HEADERS and not HAVE_STRING_H */
+#if HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
+#if HAVE_STDINT_H
+# include <stdint.h>
+#endif
 
-#include <limits.h>
-
-#if HAVE_UINTPTR_T
-# if HAVE_INTTYPES_H
-#  include <inttypes.h>
-# else
-#  if HAVE_STDINT_H
-#   include <stdint.h>
-#  endif
-# endif
-#else
+#if ! HAVE_UINTPTR_T
 /* This isn't perfect, but it's good enough for Bison, which needs
    only to hash pointers.  */
 typedef size_t uintptr_t;
 #endif
 
 #include <xalloc.h>
-#define CALLOC(P, N) ((P) = xcalloc (N, sizeof *(P)))
-#define MALLOC(P, N) ((P) = xmalloc ((N) * sizeof *(P)))
-#define REALLOC(P, N) ((P) = xrealloc (P, (N) * sizeof *(P)))
-
-/* From xstrndup.c.  */
-char *xstrndup (const char *s, size_t n);
 
 
 /*---------------------.
 | Missing prototypes.  |
 `---------------------*/
 
-#if defined HAVE_DECL_STPCPY && !HAVE_DECL_STPCPY
-char *stpcpy (char *dest, const char *src);
-#endif
-
-#if defined HAVE_DECL_STRCHR && !HAVE_DECL_STRCHR
-char *strchr (const char *s, int c);
-#endif
-
-#if defined HAVE_DECL_STRSPN && !HAVE_DECL_STRSPN
-size_t strspn (const char *s, const char *accept);
-#endif
-
-#if defined HAVE_DECL_STRNLEN && !HAVE_DECL_STRNLEN
-size_t strnlen (const char *s, size_t maxlen);
-#endif
-
-#if defined HAVE_DECL_MEMCHR && !HAVE_DECL_MEMCHR
-void *memchr (const void *s, int c, size_t n);
-#endif
-
-#if defined HAVE_DECL_MEMRCHR && !HAVE_DECL_MEMRCHR
-void *memrchr (const void *s, int c, size_t n);
-#endif
-
+#include <stpcpy.h>
 
 
 /*-----------------.
@@ -157,12 +103,7 @@ void *memrchr (const void *s, int c, size_t n);
 | NLS.  |
 `------*/
 
-#if HAVE_LOCALE_H
-# include <locale.h>
-#endif
-#if !HAVE_SETLOCALE
-# define setlocale(Category, Locale)
-#endif
+#include <locale.h>
 
 #include <gettext.h>
 #define _(Msgid)  gettext (Msgid)
@@ -182,20 +123,16 @@ void *memrchr (const void *s, int c, size_t n);
 | Booleans.  |
 `-----------*/
 
-#if HAVE_STDBOOL_H
-# include <stdbool.h>
-#else
-typedef enum {false = 0, true = 1} bool;
-#endif
+#include <stdbool.h>
 
 
 /*-----------.
 | Obstacks.  |
 `-----------*/
 
-# define obstack_chunk_alloc xmalloc
-# define obstack_chunk_free  free
-# include <obstack.h>
+#define obstack_chunk_alloc xmalloc
+#define obstack_chunk_free  free
+#include <obstack.h>
 
 #define obstack_sgrow(Obs, Str) \
   obstack_grow (Obs, Str, strlen (Str))
@@ -234,21 +171,13 @@ do {								\
 | Extensions to use for the output files.  |
 `-----------------------------------------*/
 
-#ifdef VMS
-  /* VMS. */
-# define TAB_EXT	"_tab"
-# define OUTPUT_EXT	".output"
-#else /* ! VMS */
-# ifdef MSDOS
-   /* MS DOS. */
-#  define TAB_EXT	"_tab"
-#  define OUTPUT_EXT	".out"
-# else /* ! MSDOS */
-  /* Standard. */
-#  define TAB_EXT	".tab"
-#  define OUTPUT_EXT	".output"
-# endif /* ! MSDOS */
-#endif /* ! VMS */
+#ifndef OUTPUT_EXT
+# define OUTPUT_EXT ".output"
+#endif
+
+#ifndef TAB_EXT
+# define TAB_EXT ".tab"
+#endif
 
 #ifndef DEFAULT_TMPDIR
 # define DEFAULT_TMPDIR "/tmp"
@@ -266,7 +195,7 @@ do {						\
   for (_node = List; _node; _node = _next)	\
     {						\
       _next = _node->next;			\
-      XFREE (_node);				\
+      free (_node);				\
     }						\
 } while (0)
 
